//GHINEA MIHAI
//332CC


#define F_CPU 16000000
#define USART_BAUDRATE 31250
#define BAUD_PRESCALE (((F_CPU / (USART_BAUDRATE * 16UL))) - 1)

// Sound-banks available on the VS1053.
// 0x00 : GM1 (default)
// 0x78 : Drums
// 0x79 : GM2
#define SOUND_GM1 0x00
#define SOUND_DRUMS 0x78
#define SOUND_GM2 0x79

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <stdio.h>
#include "CapacitiveSensor.h"

#define NUM_OF_SAMPLES  1
#define CAP_THRESHOLD   50
#define NUM_KEYS 8

int t0,t1,t2,t3,t4,t5,t6,t7;
int instrument;
int flag = 0;
CapacitiveSensor keys[] = {
	CapacitiveSensor(0),
	CapacitiveSensor(1),
	CapacitiveSensor(2),
	CapacitiveSensor(3),
	CapacitiveSensor(4),
	CapacitiveSensor(5),
	CapacitiveSensor(6),
	CapacitiveSensor(7),
};

volatile unsigned long currentMillis = 0;
ISR(TIMER0_COMPA_vect)
{
	currentMillis++;
}


void init() 
{
	// initializarea celor 8 "butoane" (senzor capacitiv)
	for(int i=0; i<NUM_KEYS; ++i)
	{
		keys[i].set_CS_AutocaL_Millis(0xFFFFFFFF);
	}
}


void USART0_init()
{
    /* seteaza baud rate la 31250 */
    UBRR0H = 0;
    UBRR0L = BAUD_PRESCALE;

    /* porneste transmitatorul */
    //UCSR0B = (1<<TXEN0) | (1<<RXEN0);
    UCSR0B = (1<<TXEN0);

    /* seteaza formatul frame-ului: 8 biti de date, 1 biti de stop, paritate para */
    /*UCSR0C &= ~(1<<USBS0);
    UCSR0C |= (2<<UCSZ00);
    UCSR0C |= (1<<UPM01);*/
    UCSR0C = (1<<USBS0)|(3<<UCSZ00);
}

void USART0_transmit(char data)
{
    /* asteapta pana bufferul e gol */
    while(!(UCSR0A & (1<<UDRE0)));

    /* pune datele in buffer; transmisia va porni automat in urma scrierii */
    UDR0 = data;
}

void sendMIDI(char data)
{
  USART0_transmit(data);
}

void talkMIDI(char cmd, char data1, char data2) {
	
  sendMIDI(cmd);
  //Some commands only have one data byte. All cmds less than 0xBn have 2 data bytes 
  //(sort of: http://253.ccarh.org/handout/midiprotocol/)
  sendMIDI(data1);
  if( (cmd & 0xF0) <= 0xB0) {
    sendMIDI(data2);
  }
  
}

void noteOn(char channel, char note, char attack_velocity) {
	talkMIDI( (0x90 | channel), note, attack_velocity);
  //talkMIDI(channel, note, attack_velocity);

}

void noteOff(char channel, char note, char release_velocity) {
  talkMIDI( (0x80 | channel), note, release_velocity);
  //talkMIDI( channel, note, release_velocity);
}

void set(){
	talkMIDI(0xB0, 0, 0); //Bank select: drums
 	//talkMIDI(0xC0, instrument, 0); //Set instrument number
	instrument++;
	talkMIDI(0xC0, instrument, 0); //Set instrument number
}

int main(void)
{
	instrument = 5;
	t0=t1=t2=t3=t4=t5=t6=t7=0;
	init();
	USART0_init();
	
	DDRD |= (1 << PD7);
	PORTD &= ~(1 << PD7);//led test

	DDRD &= ~(1 << PD6);
	PORTD |= (1 << PD6);//btn
	//talkMIDI(0xB0, 0x07, 126); //0xB0 is channel message, set channel volume to near max (127)
  	//talkMIDI(0xB0, 1, 0x79); //DRUMS
  	//talkMIDI(0xC0, instrument, 0); //set instrument 13
  	talkMIDI(0xB0, 0, 0x78); //Bank select: drums
 	talkMIDI(0xC0, instrument, 0); //Set instrument number
  	//keys[i].capacitiveSensor(NUM_OF_SAMPLES) > CAP_THRESHOLD
	while(1) {
		//talkMIDI(0xC0, 13, 0); //set instrument 13
			if ((PIND & (1 << PD6)) == 0) {
				set();
				//talkMIDI(0xB0, 0x07, 126); //0xB0 is channel message, set channel volume to near max (127)
  				//talkMIDI(0xB0, 0, 0x00); //Default bank GM1
  				//talkMIDI(0xC0, instrument, 0); //set instrument 13
			}
			if(keys[0].capacitiveSensor(NUM_OF_SAMPLES) > CAP_THRESHOLD && t0 == 0){
				noteOn(1,86,120);
				t0 = 1;
			}
			if(keys[1].capacitiveSensor(NUM_OF_SAMPLES) > CAP_THRESHOLD && t1 == 0){
				noteOn(0,85,120);
				t1 = 1;
			}
			if(keys[2].capacitiveSensor(NUM_OF_SAMPLES) > CAP_THRESHOLD && t2 == 0){
				noteOn(0,84,120);
				t2 = 1;
			}
			if(keys[3].capacitiveSensor(NUM_OF_SAMPLES) > CAP_THRESHOLD && t3 == 0){
				noteOn(0,82,120);
				t3 = 1;
			}
			if(keys[4].capacitiveSensor(NUM_OF_SAMPLES) > CAP_THRESHOLD && t4 == 0){
				noteOn(0,80,120);
				t4 = 1;
			}
			if(keys[5].capacitiveSensor(NUM_OF_SAMPLES) > CAP_THRESHOLD && t5 == 0){
				noteOn(0,78,120);
				t5 = 1;
			}
			if(keys[6].capacitiveSensor(NUM_OF_SAMPLES) > CAP_THRESHOLD && t6 == 0){
				noteOn(0,76,120);
				t6 = 1;
			}
			if(keys[7].capacitiveSensor(NUM_OF_SAMPLES) > CAP_THRESHOLD && t7 == 0){
				noteOn(0,74,120);
				t7 = 1;
			}

			if(keys[0].capacitiveSensor(NUM_OF_SAMPLES) < CAP_THRESHOLD && t0 == 1){
				noteOff(0,86,0);
				t0 = 0;
			}
			if(keys[1].capacitiveSensor(NUM_OF_SAMPLES) < CAP_THRESHOLD && t1 == 1){
				noteOff(0,85,0);
				t1 = 0;
			}
			if(keys[2].capacitiveSensor(NUM_OF_SAMPLES) < CAP_THRESHOLD && t2 == 1){
				noteOff(0,84,0);
				t2 = 0;
			}
			if(keys[3].capacitiveSensor(NUM_OF_SAMPLES) < CAP_THRESHOLD && t3 == 1){
				noteOff(0,82,0);
				t3 = 0;
			}
			if(keys[4].capacitiveSensor(NUM_OF_SAMPLES) < CAP_THRESHOLD && t4 == 1){
				noteOff(0,80,0);
				t4 = 0;
			}
			if(keys[5].capacitiveSensor(NUM_OF_SAMPLES) < CAP_THRESHOLD && t5 == 1){
				noteOff(0,78,0);
				t5 = 0;
			}
			if(keys[6].capacitiveSensor(NUM_OF_SAMPLES) < CAP_THRESHOLD && t6 == 1){
				noteOff(0,76,0);
				t6 = 0;
			}
			if(keys[7].capacitiveSensor(NUM_OF_SAMPLES) < CAP_THRESHOLD && t7 == 1){
				noteOff(0,74,0);
				t7 = 0;
			}
			//_delay_ms(100);
		
    }

    //noteOn(0,76,120);
    //_delay_ms(1000	);
    //noteOff(0,76,0);


}
